@@ -17,6 +17,7 @@
 #include "gd3_tag.hpp"
 #include "s98_tag.hpp"
 #include "chips/scci/scci.h"
+#include "effect.hpp"
 #include "misc.hpp"
 
 class AbstractBank;
@@ -24,7 +25,7 @@ class AbstractBank;
 class BambooTracker
 {
 public:
-	BambooTracker(std::weak_ptr<Configuration> config);
+	explicit BambooTracker(std::weak_ptr<Configuration> config);
 
 	// Change confuguration
 	void changeConfiguration(std::weak_ptr<Configuration> config);
@@ -83,8 +84,8 @@ public:
 	void setArpeggioFMSequenceCommand(int arpNum, int cnt, int type, int data);
 	void setArpeggioFMLoops(int arpNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
 	void setArpeggioFMRelease(int arpNum, ReleaseType type, int begin);
-	void setInstrumentFMArpeggio(int instNum, int arpNum);
-	void setInstrumentFMArpeggioEnabled(int instNum, bool enabled);
+	void setInstrumentFMArpeggio(int instNum, FMOperatorType op, int arpNum);
+	void setInstrumentFMArpeggioEnabled(int instNum, FMOperatorType op, bool enabled);
 	std::vector<int> getArpeggioFMUsers(int arpNum) const;
 
 	void setPitchFMType(int ptNum, int type);
@@ -93,11 +94,11 @@ public:
 	void setPitchFMSequenceCommand(int ptNum, int cnt, int type, int data);
 	void setPitchFMLoops(int ptNum, std::vector<int> begins, std::vector<int> ends, std::vector<int> times);
 	void setPitchFMRelease(int ptNum, ReleaseType type, int begin);
-	void setInstrumentFMPitch(int instNum, int ptNum);
-	void setInstrumentFMPitchEnabled(int instNum, bool enabled);
+	void setInstrumentFMPitch(int instNum, FMOperatorType op, int ptNum);
+	void setInstrumentFMPitchEnabled(int instNum, FMOperatorType op, bool enabled);
 	std::vector<int> getPitchFMUsers(int ptNum) const;
 
-	void setInstrumentFMEnvelopeResetEnabled(int instNum, bool enabled);
+	void setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorType op, bool enabled);
 
 	//--- SSG
 	void addWaveFormSSGSequenceCommand(int wfNum, int type, int data);
@@ -168,7 +169,9 @@ public:
 	void toggleJamMode();
 	bool isJamMode() const;
 	void jamKeyOn(JamKey key);
+	void jamKeyOn(int keyNum);
 	void jamKeyOff(JamKey key);
+	void jamKeyOff(int keyNum);
 
 	// Play song
 	void startPlaySong();
@@ -186,8 +189,8 @@ public:
 
 	// Export
 	bool exportToWav(std::string file, int loopCnt, std::function<bool()> f);
-	bool exportToVgm(std::string file, bool gd3TagEnabled, GD3Tag tag, std::function<bool()> f);
-	bool exportToS98(std::string file, bool tagEnabled, S98Tag tag, std::function<bool()> f);
+	bool exportToVgm(std::string file, int target, bool gd3TagEnabled, GD3Tag tag, std::function<bool()> f);
+	bool exportToS98(std::string file, int target, bool tagEnabled, S98Tag tag, int rate, std::function<bool()> f);
 
 	// Backup
 	void backupModule(std::string file);
@@ -208,6 +211,9 @@ public:
 	void setStreamRate(int rate);
 	int getStreamDuration() const;
 	void setStreamDuration(int duration);
+	int getStreamTempo() const;
+	int getStreamSpeed() const;
+	bool getStreamGrooveEnabled() const;
 	void setMasterVolume(int percentage);
 	void setMasterVolumeFM(double dB);
 	void setMasterVolumeSSG(double dB);
@@ -229,8 +235,10 @@ public:
 	std::string getModuleComment() const;
 	void setModuleTickFrequency(unsigned int freq);
 	unsigned int getModuleTickFrequency() const;
-	void setModuleStepHighlightDistance(size_t dist);
-	size_t getModuleStepHighlightDistance() const;
+	void setModuleStepHighlight1Distance(size_t dist);
+	size_t getModuleStepHighlight1Distance() const;
+	void setModuleStepHighlight2Distance(size_t dist);
+	size_t getModuleStepHighlight2Distance() const;
 	size_t getGrooveCount() const;
 	void setGroove(int num, std::vector<int> seq);
 	void setGrooves(std::vector<std::vector<int>> seqs);
@@ -252,6 +260,9 @@ public:
 	void addSong(SongType songType, std::string title);
 	void sortSongs(std::vector<int> numbers);
 	size_t getAllStepCount(int songNum, int loopCnt) const;
+	/*----- Track -----*/
+	void setEffectDisplayWidth(int songNum, int trackNum, size_t w);
+	size_t getEffectDisplayWidth(int songNum, int trackNum) const;
 	/*----- Order -----*/
 	std::vector<OrderData> getOrderData(int songNum, int orderNum) const;
 	void setOrderPattern(int songNum, int trackNum, int orderNum, int patternNum);
@@ -322,6 +333,8 @@ public:
 	size_t getPatternSizeFromOrderNumber(int songNum, int orderNum) const;
 	void setDefaultPatternSize(int songNum, size_t size);
 	size_t getDefaultPatternSize(int songNum) const;
+	/*----- Visual -----*/
+	void getOutputHistory(int16_t* container);
 
 private:
 	CommandManager comMan_;
@@ -364,21 +377,36 @@ private:
 	void readStep();
 	void readTick(int rest);
 
+	void checkFMDelayEventsInTick(Step& step, int ch);
 	void readTickFMForNoteDelay(Step& step, int ch);
 	void envelopeResetEffectFM(Step& step, int ch);
+	void checkSSGDelayEventsInTick(Step& step, int ch);
+	void checkDrumDelayEventsInTick(Step& step, int ch);
 
-	void clearDelayCounts();
+	void clearEffectQueues();
+	void clearNoteDelayCounts();
+	void clearDelayBeyondStepCounts();
+	void clearFMDelayBeyondStepCounts(int ch);
+	void clearSSGDelayBeyondStepCounts(int ch);
+	void clearDrumDelayBeyondStepCounts(int ch);
+	void updateDelayEventCounts();
 
-	bool readFMStep(Step& step, int ch, bool isSkippedSpecial = false);
-	bool readSSGStep(Step& step, int ch, bool isSkippedSpecial = false);
-	bool readDrumStep(Step& step, int ch, bool isSkippedSpecial = false);
+	bool readFMStep(Step& step, int ch);
+	bool readFMEventsInStep(Step& step, int ch, bool calledByNoteDelay = false);
+	bool readSSGStep(Step& step, int ch);
+	bool readSSGEventsInStep(Step& step, int ch, bool calledByNoteDelay = false);
+	bool readDrumStep(Step& step, int ch);
+	bool readDrumEventsInStep(Step& step, int ch, bool calledByNoteDelay = false);
 
-	bool readFMEffect(int ch, std::string id, int value, bool isSkippedSpecial = false);
-	bool readSSGEffect(int ch, std::string id, int value, bool isSkippedSpecial = false);
-	bool readDrumEffect(int ch, std::string id, int value, bool isSkippedSpecial = false);
-	bool readFMSpecialEffect(int ch, std::string id, int value);
-	bool readSSGSpecialEffect(int ch, std::string id, int value);
-	bool readDrumSpecialEffect(int ch, std::string id, int value);
+	std::vector<std::vector<Effect>> keyOnBasedEffsFM_, stepBeginBasedEffsFM_, stepEndBasedEffsFM_;
+	std::vector<std::vector<Effect>> keyOnBasedEffsSSG_, stepBeginBasedEffsSSG_, stepEndBasedEffsSSG_;
+	std::vector<std::vector<Effect>> keyOnBasedEffsDrum_, stepBeginBasedEffsDrum_, stepEndBasedEffsDrum_;
+	bool setEffectToQueueFM(int ch, Effect eff);
+	bool readFMEffectFromQueue(int ch);
+	bool setEffectToQueueSSG(int ch, Effect eff);
+	bool readSSGEffectFromQueue(int ch);
+	bool setEffectToQueueDrum(int ch, Effect eff);
+	bool readDrumEffectFromQueue(int ch);
 
 	bool effPositionJump(int nextOrder);
 	void effTrackEnd();
