@@ -1,4 +1,5 @@
 #include "configuration.hpp"
+#include "jam_manager.hpp"
 
 Configuration::Configuration()
 {
@@ -27,10 +28,13 @@ Configuration::Configuration()
 	moveCursorToRight_ = false;
 	retrieveChannelState_ = false;
 	enableTranslation_ = true;
+	showFMDetuneSigned_ = false;
+	showWaveVisual_ = true;
 
 	// Edit settings
 	pageJumpLength_ = 4;
 	editableStep_ = 1;
+	keyRepetision_ = true;
 
 	// Keys
 	keyOffKey_ = u8"-";
@@ -39,19 +43,23 @@ Configuration::Configuration()
 	echoKey_ = u8"^";
 	noteEntryLayout_ = QWERTY;
 
-	// Sonud //
+	// Sound //
 	sndDevice_ = u8"";
 	useSCCI_ = false;
+	emulator_ = 0;
 	sampleRate_ = 44100;
 	bufferLength_ = 40;
 
+	// Midi //
+	midiInPort_ = u8"";
+
 	// Mixer //
 	mixerVolumeMaster_ = 100;
 	mixerVolumeFM_ = 0;
 	mixerVolumeSSG_ = 0;
 
 	// Input //
-	fmEnvelopeTextMap_ = {
+	fmEnvelopeTexts_ = {
 		{
 			"PMD",
 			std::vector<FMEnvelopeTextType>({
@@ -465,6 +473,114 @@ Configuration::Configuration()
 			})
 		}
 	};
+
+	// Layouts
+	const std::map<std::string, JamKey> mappingQWERTY = {
+		{u8"Z",         JamKey::LOW_C},
+		{u8"S",         JamKey::LOW_CS},
+		{u8"X",         JamKey::LOW_D},
+		{u8"D",         JamKey::LOW_DS},
+		{u8"C",         JamKey::LOW_E},
+		{u8"V",         JamKey::LOW_F},
+		{u8"G",         JamKey::LOW_FS},
+		{u8"B",         JamKey::LOW_G},
+		{u8"H",         JamKey::LOW_GS},
+		{u8"N",         JamKey::LOW_A},
+		{u8"J",         JamKey::LOW_AS},
+		{u8"M",         JamKey::LOW_B},
+		{u8",",         JamKey::LOW_C_H},
+		{u8"L",         JamKey::LOW_CS_H},
+		{u8".",         JamKey::LOW_D_H},
+
+		{u8"Q",         JamKey::HIGH_C},
+		{u8"2",         JamKey::HIGH_CS},
+		{u8"W",         JamKey::HIGH_D},
+		{u8"3",         JamKey::HIGH_DS},
+		{u8"E",         JamKey::HIGH_E},
+		{u8"R",         JamKey::HIGH_F},
+		{u8"5",         JamKey::HIGH_FS},
+		{u8"T",         JamKey::HIGH_G},
+		{u8"6",         JamKey::HIGH_GS},
+		{u8"Y",         JamKey::HIGH_A},
+		{u8"7",         JamKey::HIGH_AS},
+		{u8"U",         JamKey::HIGH_B},
+		{u8"I",         JamKey::HIGH_C_H},
+		{u8"9",         JamKey::HIGH_CS_H},
+		{u8"O",         JamKey::HIGH_D_H},
+	};
+	const std::map<std::string, JamKey> mappingQWERTZ = {
+		{u8"Y",         JamKey::LOW_C},
+		{u8"S",         JamKey::LOW_CS},
+		{u8"X",         JamKey::LOW_D},
+		{u8"D",         JamKey::LOW_DS},
+		{u8"C",         JamKey::LOW_E},
+		{u8"V",         JamKey::LOW_F},
+		{u8"G",         JamKey::LOW_FS},
+		{u8"B",         JamKey::LOW_G},
+		{u8"H",         JamKey::LOW_GS},
+		{u8"N",         JamKey::LOW_A},
+		{u8"J",         JamKey::LOW_AS},
+		{u8"M",         JamKey::LOW_B},
+		{u8",",         JamKey::LOW_C_H},
+		{u8"L",         JamKey::LOW_CS_H},
+		{u8".",         JamKey::LOW_D_H},
+
+		{u8"Q",         JamKey::HIGH_C},
+		{u8"2",         JamKey::HIGH_CS},
+		{u8"W",         JamKey::HIGH_D},
+		{u8"3",         JamKey::HIGH_DS},
+		{u8"E",         JamKey::HIGH_E},
+		{u8"R",         JamKey::HIGH_F},
+		{u8"5",         JamKey::HIGH_FS},
+		{u8"T",         JamKey::HIGH_G},
+		{u8"6",         JamKey::HIGH_GS},
+		{u8"Z",         JamKey::HIGH_A},
+		{u8"7",         JamKey::HIGH_AS},
+		{u8"U",         JamKey::HIGH_B},
+		{u8"I",         JamKey::HIGH_C_H},
+		{u8"9",         JamKey::HIGH_CS_H},
+		{u8"O",         JamKey::HIGH_D_H},
+	};
+	const std::map<std::string, JamKey> mappingAZERTY = {
+		{u8"W",         JamKey::LOW_C},
+		{u8"S",         JamKey::LOW_CS},
+		{u8"X",         JamKey::LOW_D},
+		{u8"D",         JamKey::LOW_DS},
+		{u8"C",         JamKey::LOW_E},
+		{u8"V",         JamKey::LOW_F},
+		{u8"G",         JamKey::LOW_FS},
+		{u8"B",         JamKey::LOW_G},
+		{u8"H",         JamKey::LOW_GS},
+		{u8"N",         JamKey::LOW_A},
+		{u8"J",         JamKey::LOW_AS},
+		{u8",",         JamKey::LOW_B},
+		{u8";",         JamKey::LOW_C_H},
+		{u8"L",         JamKey::LOW_CS_H},
+		{u8".",         JamKey::LOW_D_H},
+
+		{u8"A",         JamKey::HIGH_C},
+		{u8"É",         JamKey::HIGH_CS},   //é - \xc9
+		{u8"Z",         JamKey::HIGH_D},
+		{u8"\"",        JamKey::HIGH_DS},
+		{u8"E",         JamKey::HIGH_E},
+		{u8"R",         JamKey::HIGH_F},
+		{u8"(",         JamKey::HIGH_FS},
+		{u8"T",         JamKey::HIGH_G},
+		{u8"-",         JamKey::HIGH_GS},
+		{u8"Y",         JamKey::HIGH_A},
+		{u8"È",         JamKey::HIGH_AS},   //è - \xc8
+		{u8"U",         JamKey::HIGH_B},
+		{u8"I",         JamKey::HIGH_C_H},
+		{u8"Ç",         JamKey::HIGH_CS_H}, //ç - \xc7
+		{u8"O",         JamKey::HIGH_D_H},
+	};
+	mappingCustom = {};
+	mappingLayouts = {
+		{Custom, mappingCustom},
+		{QWERTY, mappingQWERTY},
+		{QWERTZ, mappingQWERTZ},
+		{AZERTY, mappingAZERTY}
+	};
 }
 
 // Internal //
@@ -679,6 +795,26 @@ bool Configuration::getEnableTranslation() const
 	return enableTranslation_;
 }
 
+void Configuration::setShowFMDetuneAsSigned(bool enabled)
+{
+	showFMDetuneSigned_ = enabled;
+}
+
+bool Configuration::getShowFMDetuneAsSigned() const
+{
+	return showFMDetuneSigned_;
+}
+
+void Configuration::setShowWaveVisual(bool enabled)
+{
+	showWaveVisual_ = enabled;
+}
+
+bool Configuration::getShowWaveVisual() const
+{
+	return showWaveVisual_;
+}
+
 // Edit settings
 void Configuration::setPageJumpLength(size_t length)
 {
@@ -700,6 +836,16 @@ size_t Configuration::getEditableStep() const
 	return editableStep_;
 }
 
+void Configuration::setKeyRepetition(bool enabled)
+{
+	keyRepetision_ = enabled;
+}
+
+bool Configuration::getKeyRepetition() const
+{
+	return keyRepetision_;
+}
+
 // Keys
 void Configuration::setKeyOffKey(std::string key)
 {
@@ -751,6 +897,16 @@ Configuration::KeyboardLayout Configuration::getNoteEntryLayout() const
 	return noteEntryLayout_;
 }
 
+void Configuration::setCustomLayoutKeys(std::map<std::string, JamKey> mapping)
+{
+	mappingLayouts[KeyboardLayout::Custom] = mapping;
+}
+
+std::map<std::string, JamKey> Configuration::getCustomLayoutKeys() const
+{
+	return mappingLayouts.at(KeyboardLayout::Custom);
+}
+
 // Sound //
 void Configuration::setSoundDevice(std::string device)
 {
@@ -772,6 +928,16 @@ bool Configuration::getUseSCCI() const
 	return useSCCI_;
 }
 
+void Configuration::setEmulator(int emulator)
+{
+	emulator_ = emulator;
+}
+
+int Configuration::getEmulator() const
+{
+	return emulator_;
+}
+
 void Configuration::setSampleRate(uint32_t rate)
 {
 	sampleRate_ = rate;
@@ -792,6 +958,17 @@ size_t Configuration::getBufferLength() const
 	return bufferLength_;
 }
 
+// Midi //
+void Configuration::setMidiInputPort(const std::string &port)
+{
+	midiInPort_ = port;
+}
+
+std::string Configuration::getMidiInputPort() const
+{
+	return midiInPort_;
+}
+
 // Mixer //
 void Configuration::setMixerVolumeMaster(int percentage)
 {
@@ -824,12 +1001,13 @@ double Configuration::getMixerVolumeSSG() const
 }
 
 // Input //
-void Configuration::setFMEnvelopeTextMap(std::map<std::string, std::vector<FMEnvelopeTextType>> map)
+void Configuration::setFMEnvelopeTexts(std::vector<FMEnvelopeText> texts)
 {
-	fmEnvelopeTextMap_ = map;
+	fmEnvelopeTexts_ = texts;
 }
 
-std::map<std::string, std::vector<FMEnvelopeTextType>> Configuration::getFMEnvelopeTextMap() const
+std::vector<FMEnvelopeText> Configuration::getFMEnvelopeTexts() const
 {
-	return fmEnvelopeTextMap_;
+	return fmEnvelopeTexts_;
 }
+
