@@ -396,6 +396,12 @@ InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 					 this, &InstrumentEditorFMForm::onOperatorSequenceTypeChanged);
 
 	//========== Arpeggio ==========//
+	ui->arpOpComboBox->addItem("All", static_cast<int>(FMOperatorType::All));
+	ui->arpOpComboBox->addItem("Op1", static_cast<int>(FMOperatorType::Op1));
+	ui->arpOpComboBox->addItem("Op2", static_cast<int>(FMOperatorType::Op2));
+	ui->arpOpComboBox->addItem("Op3", static_cast<int>(FMOperatorType::Op3));
+	ui->arpOpComboBox->addItem("Op4", static_cast<int>(FMOperatorType::Op4));
+
 	ui->arpEditor->setMaximumDisplayedRowCount(15);
 	ui->arpEditor->setDefaultRow(48);
 	ui->arpEditor->setLabelDiaplayMode(true);
@@ -403,8 +409,9 @@ InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 		int d = i - 48;
 		auto text = QString::number(d);
 		if (d > 0) text = "+" + text;
-		ui->arpEditor->AddRow(text);
+		ui->arpEditor->AddRow(text, false);
 	}
+	ui->arpEditor->autoFitLabelWidth();
 	ui->arpEditor->setUpperRow(55);
 	ui->arpEditor->setMMLDisplay0As(-48);
 
@@ -459,8 +466,17 @@ InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 	// Leave Before Qt5.7.0 style due to windows xp
 	QObject::connect(ui->arpTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
 					 this, &InstrumentEditorFMForm::onArpeggioTypeChanged);
+	// Leave Before Qt5.7.0 style due to windows xp
+	QObject::connect(ui->arpOpComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+					 this, &InstrumentEditorFMForm::onArpeggioOperatorChanged);
 
 	//========== Pitch ==========//
+	ui->ptOpComboBox->addItem("All", static_cast<int>(FMOperatorType::All));
+	ui->ptOpComboBox->addItem("Op1", static_cast<int>(FMOperatorType::Op1));
+	ui->ptOpComboBox->addItem("Op2", static_cast<int>(FMOperatorType::Op2));
+	ui->ptOpComboBox->addItem("Op3", static_cast<int>(FMOperatorType::Op3));
+	ui->ptOpComboBox->addItem("Op4", static_cast<int>(FMOperatorType::Op4));
+
 	ui->ptEditor->setMaximumDisplayedRowCount(15);
 	ui->ptEditor->setDefaultRow(127);
 	ui->ptEditor->setLabelDiaplayMode(true);
@@ -468,8 +484,9 @@ InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 		int d = i - 127;
 		auto text = QString::number(d);
 		if (d > 0) text = "+" + text;
-		ui->ptEditor->AddRow(text);
+		ui->ptEditor->AddRow(text, false);
 	}
+	ui->ptEditor->autoFitLabelWidth();
 	ui->ptEditor->setUpperRow(134);
 	ui->ptEditor->setMMLDisplay0As(-127);
 
@@ -523,6 +540,46 @@ InstrumentEditorFMForm::InstrumentEditorFMForm(int num, QWidget *parent) :
 	// Leave Before Qt5.7.0 style due to windows xp
 	QObject::connect(ui->ptTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
 					 this, &InstrumentEditorFMForm::onPitchTypeChanged);
+	// Leave Before Qt5.7.0 style due to windows xp
+	QObject::connect(ui->ptOpComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+					 this, &InstrumentEditorFMForm::onPitchOperatorChanged);
+
+	//========== Others ==========//
+	QObject::connect(ui->envResetCheckBox, &QCheckBox::stateChanged,
+					 this, [&](int state) {
+		if (!isIgnoreEvent_) {
+			bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, FMOperatorType::All, (state == Qt::Checked));
+			emit modified();
+		}
+	});
+	QObject::connect(ui->envResetOp1CheckBox, &QCheckBox::stateChanged,
+					 this, [&](int state) {
+		if (!isIgnoreEvent_) {
+			bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, FMOperatorType::Op1, (state == Qt::Checked));
+			emit modified();
+		}
+	});
+	QObject::connect(ui->envResetOp2CheckBox, &QCheckBox::stateChanged,
+					 this, [&](int state) {
+		if (!isIgnoreEvent_) {
+			bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, FMOperatorType::Op2, (state == Qt::Checked));
+			emit modified();
+		}
+	});
+	QObject::connect(ui->envResetOp3CheckBox, &QCheckBox::stateChanged,
+					 this, [&](int state) {
+		if (!isIgnoreEvent_) {
+			bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, FMOperatorType::Op3, (state == Qt::Checked));
+			emit modified();
+		}
+	});
+	QObject::connect(ui->envResetOp4CheckBox, &QCheckBox::stateChanged,
+					 this, [&](int state) {
+		if (!isIgnoreEvent_) {
+			bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, FMOperatorType::Op4, (state == Qt::Checked));
+			emit modified();
+		}
+	});
 }
 
 InstrumentEditorFMForm::~InstrumentEditorFMForm()
@@ -546,13 +603,23 @@ void InstrumentEditorFMForm::setConfiguration(std::weak_ptr<Configuration> confi
 	config_ = config;
 
 	std::vector<QString> names;
-	for (auto pair : config.lock()->getFMEnvelopeTextMap()) {
-		names.push_back(QString::fromUtf8(pair.first.c_str(), pair.first.length()));
+	for (auto texts : config.lock()->getFMEnvelopeTexts()) {
+		names.push_back(QString::fromUtf8(texts.name.c_str(), static_cast<int>(texts.name.length())));
 	}
 	ui->op1Table->setEnvelopeSetNames(names);
 	ui->op2Table->setEnvelopeSetNames(names);
 	ui->op3Table->setEnvelopeSetNames(names);
 	ui->op4Table->setEnvelopeSetNames(names);
+
+	updateConfigurationForDisplay();
+}
+
+void InstrumentEditorFMForm::updateConfigurationForDisplay()
+{
+	ui->op1Table->setDTDisplayType(config_.lock()->getShowFMDetuneAsSigned());
+	ui->op2Table->setDTDisplayType(config_.lock()->getShowFMDetuneAsSigned());
+	ui->op3Table->setDTDisplayType(config_.lock()->getShowFMDetuneAsSigned());
+	ui->op4Table->setDTDisplayType(config_.lock()->getShowFMDetuneAsSigned());
 }
 
 void InstrumentEditorFMForm::setColorPalette(std::shared_ptr<ColorPalette> palette)
@@ -607,7 +674,7 @@ void InstrumentEditorFMForm::updateInstrumentParameters()
 
 	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
 	auto instFM = dynamic_cast<InstrumentFM*>(inst.get());
-	auto name = QString::fromUtf8(instFM->getName().c_str(), instFM->getName().length());
+	auto name = QString::fromUtf8(instFM->getName().c_str(), static_cast<int>(instFM->getName().length()));
 	setWindowTitle(QString("%1: %2").arg(instNum_, 2, 16, QChar('0')).toUpper().arg(name));
 
 	setInstrumentEnvelopeParameters();
@@ -616,7 +683,7 @@ void InstrumentEditorFMForm::updateInstrumentParameters()
 	setInstrumentArpeggioParameters();
 	setInstrumentPitchParameters();
 
-	ui->envResetCheckBox->setChecked(instFM->getEnvelopeResetEnabled());
+	setInstrumentEnvelopeResetParameters();
 }
 
 /********** Events **********/
@@ -626,14 +693,16 @@ void InstrumentEditorFMForm::keyPressEvent(QKeyEvent *event)
 	// For jam key on
 
 	// Check keys
-	QString seq = QKeySequence(event->modifiers() | event->key()).toString();
-	if (seq == QKeySequence(QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
-											  config_.lock()->getOctaveUpKey().length())).toString()) {
+	QString seq = QKeySequence(static_cast<int>(event->modifiers()) | event->key()).toString();
+	if (seq == QKeySequence(
+				QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
+								  static_cast<int>(config_.lock()->getOctaveUpKey().length()))).toString()) {
 		emit octaveChanged(true);
 		return;
 	}
-	else if (seq == QKeySequence(QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
-												   config_.lock()->getOctaveDownKey().length())).toString()) {
+	else if (seq == QKeySequence(
+				 QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
+								   static_cast<int>(config_.lock()->getOctaveDownKey().length()))).toString()) {
 		emit octaveChanged(false);
 		return;
 	}
@@ -769,6 +838,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters()
 	ui->op1Table->setValue(Ui::FMOperatorParameter::ML, instFM->getEnvelopeParameter(FMEnvelopeParameter::ML1));
 	ui->op1Table->setValue(Ui::FMOperatorParameter::DT, instFM->getEnvelopeParameter(FMEnvelopeParameter::DT1));
 	ui->op1Table->setValue(Ui::FMOperatorParameter::SSGEG, instFM->getEnvelopeParameter(FMEnvelopeParameter::SSGEG1));
+	ui->op1Table->setGroupEnabled(instFM->getOperatorEnabled(0));
 	ui->op2Table->setValue(Ui::FMOperatorParameter::AR, instFM->getEnvelopeParameter(FMEnvelopeParameter::AR2));
 	ui->op2Table->setValue(Ui::FMOperatorParameter::DR, instFM->getEnvelopeParameter(FMEnvelopeParameter::DR2));
 	ui->op2Table->setValue(Ui::FMOperatorParameter::SR, instFM->getEnvelopeParameter(FMEnvelopeParameter::SR2));
@@ -779,6 +849,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters()
 	ui->op2Table->setValue(Ui::FMOperatorParameter::ML, instFM->getEnvelopeParameter(FMEnvelopeParameter::ML2));
 	ui->op2Table->setValue(Ui::FMOperatorParameter::DT, instFM->getEnvelopeParameter(FMEnvelopeParameter::DT2));
 	ui->op2Table->setValue(Ui::FMOperatorParameter::SSGEG, instFM->getEnvelopeParameter(FMEnvelopeParameter::SSGEG2));
+	ui->op2Table->setGroupEnabled(instFM->getOperatorEnabled(1));
 	ui->op3Table->setValue(Ui::FMOperatorParameter::AR, instFM->getEnvelopeParameter(FMEnvelopeParameter::AR3));
 	ui->op3Table->setValue(Ui::FMOperatorParameter::DR, instFM->getEnvelopeParameter(FMEnvelopeParameter::DR3));
 	ui->op3Table->setValue(Ui::FMOperatorParameter::SR, instFM->getEnvelopeParameter(FMEnvelopeParameter::SR3));
@@ -789,6 +860,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters()
 	ui->op3Table->setValue(Ui::FMOperatorParameter::ML, instFM->getEnvelopeParameter(FMEnvelopeParameter::ML3));
 	ui->op3Table->setValue(Ui::FMOperatorParameter::DT, instFM->getEnvelopeParameter(FMEnvelopeParameter::DT3));
 	ui->op3Table->setValue(Ui::FMOperatorParameter::SSGEG, instFM->getEnvelopeParameter(FMEnvelopeParameter::SSGEG3));
+	ui->op3Table->setGroupEnabled(instFM->getOperatorEnabled(2));
 	ui->op4Table->setValue(Ui::FMOperatorParameter::AR, instFM->getEnvelopeParameter(FMEnvelopeParameter::AR4));
 	ui->op4Table->setValue(Ui::FMOperatorParameter::DR, instFM->getEnvelopeParameter(FMEnvelopeParameter::DR4));
 	ui->op4Table->setValue(Ui::FMOperatorParameter::SR, instFM->getEnvelopeParameter(FMEnvelopeParameter::SR4));
@@ -799,6 +871,7 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters()
 	ui->op4Table->setValue(Ui::FMOperatorParameter::ML, instFM->getEnvelopeParameter(FMEnvelopeParameter::ML4));
 	ui->op4Table->setValue(Ui::FMOperatorParameter::DT, instFM->getEnvelopeParameter(FMEnvelopeParameter::DT4));
 	ui->op4Table->setValue(Ui::FMOperatorParameter::SSGEG, instFM->getEnvelopeParameter(FMEnvelopeParameter::SSGEG4));
+	ui->op4Table->setGroupEnabled(instFM->getOperatorEnabled(3));
 }
 
 void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(QString data)
@@ -860,62 +933,91 @@ void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(QString data)
 	}
 }
 
-void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(QString envType, QString data)
+void InstrumentEditorFMForm::setInstrumentEnvelopeParameters(int envTypeNum, QString data)
 {
-	data.replace(QRegularExpression(R"(\D+)"), ",");
-	if (data.startsWith(",")) data.remove(0, 1);
-	if (data.endsWith(",")) data.remove(data.length() - 1, 1);
-	QStringList digits = data.split(",");
+	QStringList digits;
+	QRegularExpression re(R"((-?\d+))");
+	auto it = re.globalMatch(data);
+	while (it.hasNext()) {
+		auto match = it.next();
+		digits.append(match.captured(1));
+	}
 
-	std::vector<FMEnvelopeTextType> set = config_.lock()->getFMEnvelopeTextMap().at(envType.toUtf8().toStdString());
-	if (static_cast<int>(set.size()) > digits.size()) {
-		QMessageBox::critical(this, tr("Error"), tr("Did not match the clipboard text format with %1.").arg(envType));
+	std::vector<FMEnvelopeTextType> set
+			= config_.lock()->getFMEnvelopeTexts().at(static_cast<size_t>(envTypeNum)).texts;
+	if (static_cast<int>(set.size()) != digits.size()) {
+		auto name = config_.lock()->getFMEnvelopeTexts().at(static_cast<size_t>(envTypeNum)).name;
+		QMessageBox::critical(this, tr("Error"),
+							  tr("Did not match the clipboard text format with %1.")
+							  .arg(QString::fromUtf8(name.c_str(), static_cast<int>(name.length()))));
 		return;
 	}
 
-	for (int i = 0; i < digits.size(); ++i) {
-		int d = digits[i].toInt();
-		switch (set[i]) {
-		case FMEnvelopeTextType::Skip:	break;
-		case FMEnvelopeTextType::AL:	ui->alSlider->setValue(d);	break;
-		case FMEnvelopeTextType::FB:	ui->fbSlider->setValue(d);	break;
-		case FMEnvelopeTextType::AR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::AR, d);	break;
-		case FMEnvelopeTextType::DR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::DR, d);	break;
-		case FMEnvelopeTextType::SR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::SR, d);	break;
-		case FMEnvelopeTextType::RR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::RR, d);	break;
-		case FMEnvelopeTextType::SL1:	ui->op1Table->setValue(Ui::FMOperatorParameter::SL, d);	break;
-		case FMEnvelopeTextType::TL1:	ui->op1Table->setValue(Ui::FMOperatorParameter::TL, d);	break;
-		case FMEnvelopeTextType::KS1:	ui->op1Table->setValue(Ui::FMOperatorParameter::KS, d);	break;
-		case FMEnvelopeTextType::ML1:	ui->op1Table->setValue(Ui::FMOperatorParameter::ML, d);	break;
-		case FMEnvelopeTextType::DT1:	ui->op1Table->setValue(Ui::FMOperatorParameter::DT, d);	break;
-		case FMEnvelopeTextType::AR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::AR, d);	break;
-		case FMEnvelopeTextType::DR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::DR, d);	break;
-		case FMEnvelopeTextType::SR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::SR, d);	break;
-		case FMEnvelopeTextType::RR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::RR, d);	break;
-		case FMEnvelopeTextType::SL2:	ui->op2Table->setValue(Ui::FMOperatorParameter::SL, d);	break;
-		case FMEnvelopeTextType::TL2:	ui->op2Table->setValue(Ui::FMOperatorParameter::TL, d);	break;
-		case FMEnvelopeTextType::KS2:	ui->op2Table->setValue(Ui::FMOperatorParameter::KS, d);	break;
-		case FMEnvelopeTextType::ML2:	ui->op2Table->setValue(Ui::FMOperatorParameter::ML, d);	break;
-		case FMEnvelopeTextType::DT2:	ui->op2Table->setValue(Ui::FMOperatorParameter::DT, d);	break;
-		case FMEnvelopeTextType::AR3:	ui->op4Table->setValue(Ui::FMOperatorParameter::AR, d);	break;
-		case FMEnvelopeTextType::DR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::DR, d);	break;
-		case FMEnvelopeTextType::SR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::SR, d);	break;
-		case FMEnvelopeTextType::RR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::RR, d);	break;
-		case FMEnvelopeTextType::SL3:	ui->op3Table->setValue(Ui::FMOperatorParameter::SL, d);	break;
-		case FMEnvelopeTextType::TL3:	ui->op3Table->setValue(Ui::FMOperatorParameter::TL, d);	break;
-		case FMEnvelopeTextType::KS3:	ui->op3Table->setValue(Ui::FMOperatorParameter::KS, d);	break;
-		case FMEnvelopeTextType::ML3:	ui->op3Table->setValue(Ui::FMOperatorParameter::ML, d);	break;
-		case FMEnvelopeTextType::DT3:	ui->op3Table->setValue(Ui::FMOperatorParameter::DT, d);	break;
-		case FMEnvelopeTextType::AR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::AR, d);	break;
-		case FMEnvelopeTextType::DR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::DR, d);	break;
-		case FMEnvelopeTextType::SR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::SR, d);	break;
-		case FMEnvelopeTextType::RR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::RR, d);	break;
-		case FMEnvelopeTextType::SL4:	ui->op4Table->setValue(Ui::FMOperatorParameter::SL, d);	break;
-		case FMEnvelopeTextType::TL4:	ui->op4Table->setValue(Ui::FMOperatorParameter::TL, d);	break;
-		case FMEnvelopeTextType::KS4:	ui->op4Table->setValue(Ui::FMOperatorParameter::KS, d);	break;
-		case FMEnvelopeTextType::ML4:	ui->op4Table->setValue(Ui::FMOperatorParameter::ML, d);	break;
-		case FMEnvelopeTextType::DT4:	ui->op4Table->setValue(Ui::FMOperatorParameter::DT, d);	break;
+	auto rangeCheck = [](int v, int min, int max) -> int {
+		if (v < min || max < v) throw std::out_of_range("");
+		else return v;
+	};
+	auto rangeCheckDT = [rangeCheck](int v) -> int {
+		switch (v) {
+		case -3:	v = 7;	break;
+		case -2:	v = 6;	break;
+		case -1:	v = 5;	break;
+		default:			break;
 		}
+		return rangeCheck(v, 0, 7);
+	};
+	try {
+		for (int i = 0; i < digits.size(); ++i) {
+			int d = digits[i].toInt();
+			switch (set[static_cast<size_t>(i)]) {
+			case FMEnvelopeTextType::Skip:	break;
+			case FMEnvelopeTextType::AL:	ui->alSlider->setValue(rangeCheck(d, 0, 7));	break;
+			case FMEnvelopeTextType::FB:	ui->fbSlider->setValue(rangeCheck(d, 0, 7));	break;
+			case FMEnvelopeTextType::AR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::AR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::DR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::DR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::SR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::SR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::RR1:	ui->op1Table->setValue(Ui::FMOperatorParameter::RR, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::SL1:	ui->op1Table->setValue(Ui::FMOperatorParameter::SL, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::TL1:	ui->op1Table->setValue(Ui::FMOperatorParameter::TL, rangeCheck(d, 0, 127));	break;
+			case FMEnvelopeTextType::KS1:	ui->op1Table->setValue(Ui::FMOperatorParameter::KS, rangeCheck(d, 0, 3));	break;
+			case FMEnvelopeTextType::ML1:	ui->op1Table->setValue(Ui::FMOperatorParameter::ML, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::DT1:	ui->op1Table->setValue(Ui::FMOperatorParameter::DT, rangeCheckDT(d));	break;
+			case FMEnvelopeTextType::AR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::AR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::DR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::DR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::SR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::SR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::RR2:	ui->op2Table->setValue(Ui::FMOperatorParameter::RR, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::SL2:	ui->op2Table->setValue(Ui::FMOperatorParameter::SL, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::TL2:	ui->op2Table->setValue(Ui::FMOperatorParameter::TL, rangeCheck(d, 0, 127));	break;
+			case FMEnvelopeTextType::KS2:	ui->op2Table->setValue(Ui::FMOperatorParameter::KS, rangeCheck(d, 0, 3));	break;
+			case FMEnvelopeTextType::ML2:	ui->op2Table->setValue(Ui::FMOperatorParameter::ML, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::DT2:	ui->op2Table->setValue(Ui::FMOperatorParameter::DT, rangeCheckDT(d));	break;
+			case FMEnvelopeTextType::AR3:	ui->op4Table->setValue(Ui::FMOperatorParameter::AR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::DR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::DR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::SR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::SR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::RR3:	ui->op3Table->setValue(Ui::FMOperatorParameter::RR, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::SL3:	ui->op3Table->setValue(Ui::FMOperatorParameter::SL, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::TL3:	ui->op3Table->setValue(Ui::FMOperatorParameter::TL, rangeCheck(d, 0, 127));	break;
+			case FMEnvelopeTextType::KS3:	ui->op3Table->setValue(Ui::FMOperatorParameter::KS, rangeCheck(d, 0, 4));	break;
+			case FMEnvelopeTextType::ML3:	ui->op3Table->setValue(Ui::FMOperatorParameter::ML, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::DT3:	ui->op3Table->setValue(Ui::FMOperatorParameter::DT, rangeCheckDT(d));	break;
+			case FMEnvelopeTextType::AR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::AR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::DR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::DR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::SR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::SR, rangeCheck(d, 0, 31));	break;
+			case FMEnvelopeTextType::RR4:	ui->op4Table->setValue(Ui::FMOperatorParameter::RR, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::SL4:	ui->op4Table->setValue(Ui::FMOperatorParameter::SL, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::TL4:	ui->op4Table->setValue(Ui::FMOperatorParameter::TL, rangeCheck(d, 0, 127));	break;
+			case FMEnvelopeTextType::KS4:	ui->op4Table->setValue(Ui::FMOperatorParameter::KS, rangeCheck(d, 0, 4));	break;
+			case FMEnvelopeTextType::ML4:	ui->op4Table->setValue(Ui::FMOperatorParameter::ML, rangeCheck(d, 0, 15));	break;
+			case FMEnvelopeTextType::DT4:	ui->op4Table->setValue(Ui::FMOperatorParameter::DT, rangeCheckDT(d));	break;
+			}
+		}
+	}
+	catch (...) {
+		auto name = config_.lock()->getFMEnvelopeTexts().at(static_cast<size_t>(envTypeNum)).name;
+		QMessageBox::critical(this, tr("Error"),
+							  tr("Did not match the clipboard text format with %1.")
+							  .arg(QString::fromUtf8(name.c_str(), static_cast<int>(name.length()))));
+		return;
 	}
 }
 
@@ -1187,9 +1289,9 @@ void InstrumentEditorFMForm::pasteEnvelope()
 	setInstrumentEnvelopeParameters(data);
 }
 
-void InstrumentEditorFMForm::pasteEnvelopeFrom(QString type)
+void InstrumentEditorFMForm::pasteEnvelopeFrom(int typenum)
 {
-	setInstrumentEnvelopeParameters(type, QApplication::clipboard()->text());
+	setInstrumentEnvelopeParameters(typenum, QApplication::clipboard()->text());
 }
 
 void InstrumentEditorFMForm::copyOperator(int opNum)
@@ -1234,10 +1336,14 @@ void InstrumentEditorFMForm::on_envGroupBox_customContextMenuRequested(const QPo
 	QObject::connect(paste, &QAction::triggered, this, &InstrumentEditorFMForm::pasteEnvelope);
 	paste->setEnabled(QApplication::clipboard()->text().startsWith("FM_ENVELOPE:"));
 	QMenu* pasteFrom = menu.addMenu(tr("Paste envelope From"));
-	for (auto pair : config_.lock()->getFMEnvelopeTextMap())
-		pasteFrom->addAction(QString::fromUtf8(pair.first.c_str(), pair.first.length()));
+	std::vector<FMEnvelopeText> textsSet = config_.lock()->getFMEnvelopeTexts();
+	for (size_t i = 0; i < textsSet.size(); ++i) {
+		QAction* act = pasteFrom->addAction(
+						   QString::fromUtf8(textsSet[i].name.c_str(), static_cast<int>(textsSet[i].name.length())));
+		act->setData(static_cast<int>(i));
+	}
 	QObject::connect(pasteFrom, &QMenu::triggered,
-					 this, [&](QAction* action) { pasteEnvelopeFrom(action->text()); });
+					 this, [&](QAction* action) { pasteEnvelopeFrom(action->data().toInt()); });
 
 	menu.exec(globalPos);
 }
@@ -1441,8 +1547,9 @@ void InstrumentEditorFMForm::setOperatorSequenceEditor()
 	case FMEnvelopeParameter::DT4:
 		ui->opSeqEditor->setMaximumDisplayedRowCount(8);
 		for (int i = 0; i < 8; ++i) {
-			ui->opSeqEditor->AddRow(QString::number(i));
+			ui->opSeqEditor->AddRow(QString::number(i), false);
 		}
+		ui->opSeqEditor->autoFitLabelWidth();
 		ui->opSeqEditor->setUpperRow(7);
 		break;
 	case FMEnvelopeParameter::AR1:
@@ -1532,9 +1639,7 @@ void InstrumentEditorFMForm::onOperatorSequenceTypeChanged(int type)
 {
 	Q_UNUSED(type)
 
-	if (!isIgnoreEvent_) {
-		setInstrumentOperatorSequenceParameters();
-	}
+	if (!isIgnoreEvent_) setInstrumentOperatorSequenceParameters();
 }
 
 void InstrumentEditorFMForm::on_opSeqEditGroupBox_toggled(bool arg1)
@@ -1562,6 +1667,11 @@ void InstrumentEditorFMForm::on_opSeqNumSpinBox_valueChanged(int arg1)
 }
 
 //--- Arpeggio
+FMOperatorType InstrumentEditorFMForm::getArpeggioOperator() const
+{
+	return static_cast<FMOperatorType>(ui->arpOpComboBox->currentData(Qt::UserRole).toInt());
+}
+
 void InstrumentEditorFMForm::setInstrumentArpeggioParameters()
 {
 	Ui::EventGuard ev(isIgnoreEvent_);
@@ -1569,18 +1679,20 @@ void InstrumentEditorFMForm::setInstrumentArpeggioParameters()
 	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
 	auto instFM = dynamic_cast<InstrumentFM*>(inst.get());
 
-	ui->arpNumSpinBox->setValue(instFM->getArpeggioNumber());
+	FMOperatorType param = getArpeggioOperator();
+
+	ui->arpNumSpinBox->setValue(instFM->getArpeggioNumber(param));
 	ui->arpEditor->clearData();
-	for (auto& com : instFM->getArpeggioSequence()) {
+	for (auto& com : instFM->getArpeggioSequence(param)) {
 		ui->arpEditor->addSequenceCommand(com.type);
 	}
-	for (auto& l : instFM->getArpeggioLoops()) {
+	for (auto& l : instFM->getArpeggioLoops(param)) {
 		ui->arpEditor->addLoop(l.begin, l.end, l.times);
 	}
-	ui->arpEditor->setRelease(convertReleaseTypeForUI(instFM->getArpeggioRelease().type),
-							  instFM->getArpeggioRelease().begin);
-	ui->arpTypeComboBox->setCurrentIndex(instFM->getArpeggioType());
-	if (instFM->getArpeggioEnabled()) {
+	ui->arpEditor->setRelease(convertReleaseTypeForUI(instFM->getArpeggioRelease(param).type),
+							  instFM->getArpeggioRelease(param).begin);
+	ui->arpTypeComboBox->setCurrentIndex(instFM->getArpeggioType(param));
+	if (instFM->getArpeggioEnabled(param)) {
 		ui->arpEditGroupBox->setChecked(true);
 		onArpeggioNumberChanged();
 	}
@@ -1611,6 +1723,13 @@ void InstrumentEditorFMForm::onArpeggioParameterChanged(int tnNum)
 	}
 }
 
+void InstrumentEditorFMForm::onArpeggioOperatorChanged(int op)
+{
+	Q_UNUSED(op)
+
+	if (!isIgnoreEvent_) setInstrumentArpeggioParameters();
+}
+
 void InstrumentEditorFMForm::onArpeggioTypeChanged(int index)
 {
 	Q_UNUSED(index)
@@ -1644,7 +1763,7 @@ void InstrumentEditorFMForm::onArpeggioTypeChanged(int index)
 void InstrumentEditorFMForm::on_arpEditGroupBox_toggled(bool arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentFMArpeggioEnabled(instNum_, arg1);
+		bt_.lock()->setInstrumentFMArpeggioEnabled(instNum_, getArpeggioOperator(), arg1);
 		setInstrumentArpeggioParameters();
 		emit arpeggioNumberChanged();
 		emit modified();
@@ -1656,7 +1775,7 @@ void InstrumentEditorFMForm::on_arpEditGroupBox_toggled(bool arg1)
 void InstrumentEditorFMForm::on_arpNumSpinBox_valueChanged(int arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentFMArpeggio(instNum_, arg1);
+		bt_.lock()->setInstrumentFMArpeggio(instNum_, getArpeggioOperator(), arg1);
 		setInstrumentArpeggioParameters();
 		emit arpeggioNumberChanged();
 		emit modified();
@@ -1666,6 +1785,11 @@ void InstrumentEditorFMForm::on_arpNumSpinBox_valueChanged(int arg1)
 }
 
 //--- Pitch
+FMOperatorType InstrumentEditorFMForm::getPitchOperator() const
+{
+	return static_cast<FMOperatorType>(ui->ptOpComboBox->currentData(Qt::UserRole).toInt());
+}
+
 void InstrumentEditorFMForm::setInstrumentPitchParameters()
 {
 	Ui::EventGuard ev(isIgnoreEvent_);
@@ -1673,23 +1797,25 @@ void InstrumentEditorFMForm::setInstrumentPitchParameters()
 	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
 	auto instFM = dynamic_cast<InstrumentFM*>(inst.get());
 
-	ui->ptNumSpinBox->setValue(instFM->getPitchNumber());
+	FMOperatorType param = getPitchOperator();
+
+	ui->ptNumSpinBox->setValue(instFM->getPitchNumber(param));
 	ui->ptEditor->clearData();
-	for (auto& com : instFM->getPitchSequence()) {
+	for (auto& com : instFM->getPitchSequence(param)) {
 		ui->ptEditor->addSequenceCommand(com.type);
 	}
-	for (auto& l : instFM->getPitchLoops()) {
+	for (auto& l : instFM->getPitchLoops(param)) {
 		ui->ptEditor->addLoop(l.begin, l.end, l.times);
 	}
-	ui->ptEditor->setRelease(convertReleaseTypeForUI(instFM->getPitchRelease().type),
-							 instFM->getPitchRelease().begin);
+	ui->ptEditor->setRelease(convertReleaseTypeForUI(instFM->getPitchRelease(param).type),
+							 instFM->getPitchRelease(param).begin);
 	for (int i = 0; i < ui->ptTypeComboBox->count(); ++i) {
-		if (ui->ptTypeComboBox->itemData(i, Qt::UserRole).toInt() == instFM->getPitchType()) {
+		if (ui->ptTypeComboBox->itemData(i, Qt::UserRole).toInt() == instFM->getPitchType(param)) {
 			ui->ptTypeComboBox->setCurrentIndex(i);
 			break;
 		}
 	}
-	if (instFM->getPitchEnabled()) {
+	if (instFM->getPitchEnabled(param)) {
 		ui->ptEditGroupBox->setChecked(true);
 		onPitchNumberChanged();
 	}
@@ -1720,6 +1846,13 @@ void InstrumentEditorFMForm::onPitchParameterChanged(int tnNum)
 	}
 }
 
+void InstrumentEditorFMForm::onPitchOperatorChanged(int op)
+{
+	Q_UNUSED(op)
+
+	if (!isIgnoreEvent_) setInstrumentPitchParameters();
+}
+
 void InstrumentEditorFMForm::onPitchTypeChanged(int index)
 {
 	Q_UNUSED(index)
@@ -1735,7 +1868,7 @@ void InstrumentEditorFMForm::onPitchTypeChanged(int index)
 void InstrumentEditorFMForm::on_ptEditGroupBox_toggled(bool arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentFMPitchEnabled(instNum_, arg1);
+		bt_.lock()->setInstrumentFMPitchEnabled(instNum_, getPitchOperator(), arg1);
 		setInstrumentPitchParameters();
 		emit pitchNumberChanged();
 		emit modified();
@@ -1747,7 +1880,7 @@ void InstrumentEditorFMForm::on_ptEditGroupBox_toggled(bool arg1)
 void InstrumentEditorFMForm::on_ptNumSpinBox_valueChanged(int arg1)
 {
 	if (!isIgnoreEvent_) {
-		bt_.lock()->setInstrumentFMPitch(instNum_, arg1);
+		bt_.lock()->setInstrumentFMPitch(instNum_, getPitchOperator(), arg1);
 		setInstrumentPitchParameters();
 		emit pitchNumberChanged();
 		emit modified();
@@ -1756,15 +1889,17 @@ void InstrumentEditorFMForm::on_ptNumSpinBox_valueChanged(int arg1)
 	onPitchNumberChanged();
 }
 
-//========== Else ==========//
-/********** Slots **********/
-void InstrumentEditorFMForm::on_envResetCheckBox_stateChanged(int arg1)
+//========== Others ==========//
+void InstrumentEditorFMForm::setInstrumentEnvelopeResetParameters()
 {
-	if (arg1 == Qt::Checked) {
-		bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, true);
-	}
-	else {
-		bt_.lock()->setInstrumentFMEnvelopeResetEnabled(instNum_, false);
-	}
-	emit modified();
+	Ui::EventGuard ev(isIgnoreEvent_);
+
+	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
+	auto instFM = dynamic_cast<InstrumentFM*>(inst.get());
+
+	ui->envResetCheckBox->setChecked(instFM->getEnvelopeResetEnabled(FMOperatorType::All));
+	ui->envResetOp1CheckBox->setChecked(instFM->getEnvelopeResetEnabled(FMOperatorType::Op1));
+	ui->envResetOp2CheckBox->setChecked(instFM->getEnvelopeResetEnabled(FMOperatorType::Op2));
+	ui->envResetOp3CheckBox->setChecked(instFM->getEnvelopeResetEnabled(FMOperatorType::Op3));
+	ui->envResetOp4CheckBox->setChecked(instFM->getEnvelopeResetEnabled(FMOperatorType::Op4));
 }
