@@ -23,8 +23,10 @@
 #include "audio_stream.hpp"
 #include "gui/instrument_editor/instrument_form_manager.hpp"
 #include "gui/color_palette.hpp"
-/*#include "timer.hpp"*/
+#include "timer.hpp"
 #include "gui/file_history.hpp"
+#include "gui/effect_list_dialog.hpp"
+#include "gui/keyboard_shortcut_list_dialog.hpp"
 
 class AbstractBank;
 
@@ -50,14 +52,21 @@ protected:
 	void moveEvent(QMoveEvent* event) override;
 	void closeEvent(QCloseEvent* event) override;
 
+	// Midi
+private:
+	static void midiThreadReceivedEvent(double delay, const uint8_t *msg, size_t len, void *userData);
+private slots:
+	void midiKeyEvent(uchar status, uchar key, uchar velocity);
+	void midiProgramEvent(uchar status, uchar program);
+
 private:
 	std::unique_ptr<Ui::MainWindow> ui;
 	std::weak_ptr<Configuration> config_;
 	std::shared_ptr<ColorPalette> palette_;
 	std::shared_ptr<BambooTracker> bt_;
 	std::shared_ptr<AudioStream> stream_;
-	std::unique_ptr<QTimer> timer_;
-	/*std::unique_ptr<Timer> timer_;*/
+	std::unique_ptr<Timer> timer_;
+	std::unique_ptr<QTimer> visualTimer_;
 	std::shared_ptr<QUndoStack> comStack_;
 	std::shared_ptr<FileHistory> fileHistory_;
 
@@ -71,7 +80,7 @@ private:
 	int findRowFromInstrumentList(int instNum);
 	void editInstrumentName();
 	void cloneInstrument();
-    void deepCloneInstrument();
+	void deepCloneInstrument();
 	void loadInstrument();
 	void saveInstrument();
 	void importInstrumentsFromBank();
@@ -99,13 +108,18 @@ private:
 
 	// Configuration change
 	void changeConfiguration();
+	void setMidiConfiguration();
 
 	// History change
 	void changeFileHistory(QString file);
 
+	// Layout decypherer
+	JamKey getJamKeyFromLayoutMapping(Qt::Key key);
+
 	void setWindowTitle();
 	void setModifiedTrue();
 	void setInitialSelectedInstrument();
+	QString getModuleFileBaseName() const;
 
 	bool isEditedPattern_, isEditedOrder_, isEditedInstList_;
 	bool isSelectedPO_;
@@ -114,7 +128,7 @@ private:
 
 	// Sub tool bar
 	QSpinBox* octave_;
-	QSpinBox* highlight_;
+	QSpinBox *highlight1_, *highlight2_;
 
 	// Status bar
 	QLabel* statusDetail_;
@@ -122,8 +136,18 @@ private:
 	QLabel* statusInst_;
 	QLabel* statusOctave_;
 	QLabel* statusIntr_;
+	QLabel* statusBpm_;
 	QLabel* statusPlayPos_;
 
+	// Dialogs
+	std::unique_ptr<EffectListDialog> effListDiag_;
+	std::unique_ptr<KeyboardShortcutListDialog> shortcutsDiag_;
+
+	// Meta methods
+	int tickEventMethod_;
+	int midiKeyEventMethod_;
+	int midiProgramEventMethod_;
+
 private slots:
 	void on_instrumentListWidget_customContextMenuRequested(const QPoint &pos);
 	void on_instrumentListWidget_itemDoubleClicked(QListWidgetItem *item);
@@ -199,13 +223,17 @@ private slots:
 	void on_actionOverwrite_triggered();
 	void onNewTickSignaled();
 	void on_actionClear_triggered();
+	void on_keyRepeatCheckBox_stateChanged(int arg1);
+	void updateVisuals();
+	void on_action_Effect_List_triggered();
+	void on_actionShortcuts_triggered();
 
 	inline bool showUndoResetWarningDialog(QString text)
 	{
 		return (QMessageBox::warning(this, "Warning",
-									text + " If you execute this command, the command history is reset.",
-									QMessageBox::Yes | QMessageBox::No,
-									QMessageBox::No) == QMessageBox::Yes);
+									 text + " If you execute this command, the command history is reset.",
+									 QMessageBox::Yes | QMessageBox::No,
+									 QMessageBox::No) == QMessageBox::Yes);
 	}
 };
 
