@@ -123,9 +123,12 @@ void OrderListEditor::setCurrentOrder(int num, int max)
 void OrderListEditor::onSongLoaded()
 {
 	ui->panel->onSongLoaded();
+	setMaximumWidth(ui->panel->maximumWidth() + ui->verticalScrollBar->width() + 2);
 	int song = bt_->getCurrentSongNumber();
-	ui->horizontalScrollBar->setMaximum(bt_->getSongStyle(song).trackAttribs.size() - 1);
-	ui->verticalScrollBar->setMaximum(bt_->getOrderSize(song) - 1);
+	ui->horizontalScrollBar->setValue(0);
+	ui->horizontalScrollBar->setMaximum(static_cast<int>(bt_->getSongStyle(song).trackAttribs.size()) - 1);
+	ui->verticalScrollBar->setValue(0);
+	ui->verticalScrollBar->setMaximum(static_cast<int>(bt_->getOrderSize(song)) - 1);
 }
 
 void OrderListEditor::onPastePressed()
