@@ -21,11 +21,22 @@ Song::Song(int number, SongType songType, std::string title, bool isUsedTempo,
 			tracks_.emplace_back(i + 6, SoundSource::SSG, i, defaultPatternSize);
 		}
 		for (int i = 0; i < 6; ++i) {
-			tracks_.emplace_back(i + 9, SoundSource::DRUM, i, defaultPatternSize);
+			tracks_.emplace_back(i + 9, SoundSource::Drum, i, defaultPatternSize);
 		}
 		break;
 	case SongType::FMEX:
-		// UNDONE: FM extend mode
+		tracks_.reserve(18);
+		for (int i = 0; i < 9; ++i) {
+			int ch = (i < 3) ? i
+							 : (i < 6) ? (i + 3) : (i - 3);
+			tracks_.emplace_back(i, SoundSource::FM, ch, defaultPatternSize);
+		}
+		for (int i = 0; i < 3; ++i) {
+			tracks_.emplace_back(i + 9, SoundSource::SSG, i, defaultPatternSize);
+		}
+		for (int i = 0; i < 6; ++i) {
+			tracks_.emplace_back(i + 12, SoundSource::Drum, i, defaultPatternSize);
+		}
 		break;
 	}
 }
@@ -124,7 +135,7 @@ std::vector<TrackAttribute> Song::getTrackAttributes() const
 
 Track& Song::getTrack(int num)
 {
-	return tracks_.at(num);
+	return tracks_.at(static_cast<size_t>(num));
 }
 
 std::vector<OrderData> Song::getOrderData(int order)
