@@ -7,6 +7,7 @@
 #include "opna.hpp"
 #include "instrument.hpp"
 #include "effect_iterator.hpp"
+#include "chips/chip_misc.h"
 #include "chips/scci/scci.h"
 #include "misc.hpp"
 
@@ -26,7 +27,7 @@ struct ToneNoise
 class OPNAController
 {
 public:
-	OPNAController(int clock, int rate, int duration);
+	OPNAController(chip::Emu emu, int clock, int rate, int duration);
 
 	// Reset and initialize
 	void reset();
@@ -40,6 +41,13 @@ public:
 
 	// Stream samples
 	void getStreamSamples(int16_t* container, size_t nSamples);
+	void getOutputHistory(int16_t* history);
+
+	enum { OutputHistorySize = 1024 };
+
+	// Chip mode
+	void setMode(SongType mode);
+	SongType getMode() const;
 
 	// Stream details
 	int getRate() const;
@@ -51,12 +59,15 @@ public:
 	// Export
 	void setExportContainer(std::shared_ptr<chip::ExportContainerInterface> cntr = nullptr);
 
-
 private:
 	std::unique_ptr<chip::OPNA> opna_;
+	SongType mode_;
 
 	void initChip();
 
+	void fillOutputHistory(const int16_t* outputs, size_t nSamples);
+	void transferReadyHistory();
+
 	/*----- FM -----*/
 public:
 	// Key on-off
@@ -91,6 +102,9 @@ public:
 	void setNoteSlideFM(int ch, int speed, int seminote);
 	void setTransposeEffectFM(int ch, int seminote);
 
+	// For state retrieve
+	void haltSequencesFM(int ch);
+
 	// Mute
 	void setMuteFMState(int ch, bool isMuteFM);
 	bool isMuteFM(int ch);
@@ -104,46 +118,51 @@ public:
 private:
 	std::shared_ptr<InstrumentFM> refInstFM_[6];
 	std::unique_ptr<EnvelopeFM> envFM_[6];
-	bool isKeyOnFM_[6];
+	bool isKeyOnFM_[9];
 	uint8_t fmOpEnables_[6];
-	std::deque<ToneDetail> baseToneFM_[6];
-	ToneDetail keyToneFM_[6];
-	int sumPitchFM_[6];
-	int baseVolFM_[6], tmpVolFM_[6];
+	std::deque<ToneDetail> baseToneFM_[9];
+	ToneDetail keyToneFM_[9];
+	int sumPitchFM_[9];
+	int baseVolFM_[9], tmpVolFM_[9];
 	/// bit0: right on/off
 	/// bit1: left on/off
 	uint8_t panFM_[6];
-	bool isMuteFM_[6];
-	bool enableEnvResetFM_[6];
+	bool isMuteFM_[9];
+	bool enableEnvResetFM_[9];
 	int lfoFreq_;
 	int lfoStartCntFM_[6];
-	bool hasPreSetTickEventFM_[6];
-	bool needToneSetFM_[6];
+	bool hasPreSetTickEventFM_[9];
+	bool needToneSetFM_[9];
 	std::map<FMEnvelopeParameter, std::unique_ptr<CommandSequence::Iterator>> opSeqItFM_[6];
-	std::unique_ptr<SequenceIteratorInterface> arpItFM_[6];
-	std::unique_ptr<CommandSequence::Iterator> ptItFM_[6];
-	bool isArpEffFM_[6];
-	int prtmFM_[6];
-	bool isTonePrtmFM_[6];
-	std::unique_ptr<WavingEffectIterator> vibItFM_[6];
-	std::unique_ptr<WavingEffectIterator> treItFM_[6];
-	int volSldFM_[6], sumVolSldFM_[6];
-	int detuneFM_[6];
-	std::unique_ptr<NoteSlideEffectIterator> nsItFM_[6];
-	int sumNoteSldFM_[6];
-	bool noteSldFMSetFlag_;
-	int transposeFM_[6];
+	std::unique_ptr<SequenceIteratorInterface> arpItFM_[9];
+	std::unique_ptr<CommandSequence::Iterator> ptItFM_[9];
+	bool isArpEffFM_[9];
+	int prtmFM_[9];
+	bool isTonePrtmFM_[9];
+	std::unique_ptr<WavingEffectIterator> vibItFM_[9];
+	std::unique_ptr<WavingEffectIterator> treItFM_[9];
+	int volSldFM_[9], sumVolSldFM_[9];
+	int detuneFM_[9];
+	std::unique_ptr<NoteSlideEffectIterator> nsItFM_[9];
+	int sumNoteSldFM_[9];
+	bool noteSldFMSetFlag_[9];
+	int transposeFM_[9];
 
 	void initFM();
 
-	uint32_t getFmChannelMask(int ch);
-	uint32_t getFMChannelOffset(int ch);
+	int toInternalFMChannel(int ch) const;
+	uint8_t getFMKeyOnOffChannelMask(int ch) const;
+	uint32_t getFMChannelOffset(int ch, bool forPitch = false) const;
+	FMOperatorType toChannelOperatorType(int ch) const;
+	std::vector<FMEnvelopeParameter> getFMEnvelopeParametersForOperator(FMOperatorType op) const;
 
-	void writeFMEnvelopeToRegistersFromInstrument(int ch);
-	void writeFMEnveropeParameterToRegister(int ch, FMEnvelopeParameter param, int value);
+	void updateFMVolume(int ch);
 
-	void writeFMLFOAllRegisters(int ch);
-	void writeFMLFORegister(int ch, FMLFOParameter param);
+	void writeFMEnvelopeToRegistersFromInstrument(int inch);
+	void writeFMEnveropeParameterToRegister(int inch, FMEnvelopeParameter param, int value);
+
+	void writeFMLFOAllRegisters(int inch);
+	void writeFMLFORegister(int inch, FMLFOParameter param);
 	void checkLFOUsed();
 
 	void setFrontFMSequences(int ch);
@@ -164,7 +183,7 @@ private:
 	inline uint8_t calculateTL(int ch, uint8_t data) const
 	{
 		int v = (tmpVolFM_[ch] == -1) ? baseVolFM_[ch] : tmpVolFM_[ch];
-		return (data > 127 - v) ? 127 : (data + v);
+		return (data > 127 - v) ? 127 : static_cast<uint8_t>(data + v);
 	}
 
 	/*----- SSG -----*/
@@ -194,6 +213,9 @@ public:
 	void setNoteSlideSSG(int ch, int speed, int seminote);
 	void setTransposeEffectSSG(int ch, int seminote);
 
+	// For state retrieve
+	void haltSequencesSSG(int ch);
+
 	// Mute
 	void setMuteSSGState(int ch, bool isMuteFM);
 	bool isMuteSSG(int ch);
@@ -219,10 +241,15 @@ private:
 	bool needEnvSetSSG_[3];
 	bool needMixSetSSG_[3];
 	bool needToneSetSSG_[3];
+	bool needSqMaskFreqSetSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> wfItSSG_[3];
-	CommandInSequence wfSSG_[3];
+	struct
+	{
+		SSGWaveFormType type;
+		int data;
+	} wfSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> envItSSG_[3];
-	CommandInSequence envSSG_[3];
+	CommandSequenceUnit envSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> tnItSSG_[3];
 	std::unique_ptr<SequenceIteratorInterface> arpItSSG_[3];
 	std::unique_ptr<CommandSequence::Iterator> ptItSSG_[3];
@@ -237,6 +264,10 @@ private:
 	int sumNoteSldSSG_[3];
 	bool noteSldSSGSetFlag_;
 	int transposeSSG_[3];
+	std::unique_ptr<int16_t[]> outputHistory_;
+	size_t outputHistoryIndex_;
+	std::unique_ptr<int16_t[]> outputHistoryReady_;
+	std::mutex outputHistoryReadyMutex_;
 
 	void initSSG();
 
@@ -258,8 +289,9 @@ private:
 
 	void setRealVolumeSSG(int ch);
 
-	inline uint8_t judgeSSEGRegisterValue(int v) {
-		return (v == -1) ? 0 : (0x08 + v);
+	inline uint8_t judgeSSEGRegisterValue(int v)
+	{
+		return (v == -1) ? 0 : (0x08 + static_cast<uint8_t>(v));
 	}
 
 	/*----- Drum -----*/
