@@ -22,17 +22,17 @@ public:
 private:
 	// Application version
 	static constexpr unsigned int appMajor			= 0;
-	static constexpr unsigned int appMinor			= 1;
-	static constexpr unsigned int appRevision		= 6;
+	static constexpr unsigned int appMinor			= 2;
+	static constexpr unsigned int appRevision		= 2;
 
 	// Module file version
 	static constexpr unsigned int modFileMajor		= 1;
-	static constexpr unsigned int modFileMinor		= 0;
+	static constexpr unsigned int modFileMinor		= 2;
 	static constexpr unsigned int modFileRevision	= 2;
 
 	// Instrument file version
 	static constexpr unsigned int instFileMajor		= 1;
-	static constexpr unsigned int instFileMinor		= 0;
+	static constexpr unsigned int instFileMinor		= 2;
 	static constexpr unsigned int instFileRevision	= 1;
 
 	Version() {}
@@ -70,9 +70,9 @@ inline std::string Version::ofInstrumentFileInString()
 
 inline uint32_t Version::toBCD(unsigned int major, unsigned int minor, unsigned int revision)
 {
-	uint32_t maj = uitobcd(major);
-	uint32_t min = uitobcd(minor);
-	uint32_t rev = uitobcd(revision);
+	uint32_t maj = uitobcd(static_cast<uint8_t>(major));
+	uint32_t min = uitobcd(static_cast<uint8_t>(minor));
+	uint32_t rev = uitobcd(static_cast<uint8_t>(revision));
 	return (maj << 16) + (min << 8) + rev;
 }
 
